/** @module : fifo_1
 *  @author : Adaptive & Secure Computing Systems (ASCS) Laboratory

 *  Copyright (c) 2018 ABAQS (ASCS/ECE/BU)
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.

 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 *  THE SOFTWARE.
 */


module fifo_1 #(
  parameter BIT_WIDTH = 32
)(
  input clock,
  input reset_n,

  // Input Side
  input write_rq,
  input [BIT_WIDTH-1:0] data_in,

  // Output Side
  input read_rq,
  output reg [BIT_WIDTH-1:0] data_out,

  output reg empty
);


always@(posedge clock or negedge reset_n) begin
  if( reset_n == 1'b0) begin
    data_out <= {BIT_WIDTH{1'b0}};
    empty <= 1'b1;
  end else begin
    case({read_rq, write_rq})
      2'b00: // idle
      begin
        empty <= empty;
        data_out <= data_out;
      end
      2'b01: // write
      begin
        empty <= 1'b0;
        data_out <= data_in;
      end
      2'b10: // read
      begin
        empty <= 1'b1;
        data_out <= data_out;
      end
      2'b11: // Simultaneous r/w (fifo acts like shift register)
      begin
        empty <= 1'b0;
        data_out<= data_in;
      end
    endcase
  end // if reset_n
end //always

endmodule
